@@ -6,6 +6,7 @@ use std::{
     collections::hash_map::Entry::{self, Vacant},
     iter::zip,
     rc::Rc,
+    sync::Arc,
 };
 
 use bit_iter::BitIter;
@@ -58,7 +59,7 @@ pub fn add_flatten_once(add: &Add) -> Option<Add> {
 #[pyfunction]
 pub fn add_collapse_scalar_inputs(add: &Add) -> Option<Add> {
     let nodes = add.nodes.iter().map(|x| x.0.clone());
-    let (scalar_inputs, non_scalar_inputs): (Vec<ScalarConstant>, Vec<Rc<Circuit>>) =
+    let (scalar_inputs, non_scalar_inputs): (Vec<ScalarConstant>, Vec<Arc<Circuit>>) =
         filter_by_variant!(nodes, Circuit, ScalarConstant, ScalarConstant);
     if scalar_inputs.len() <= 1 {
         None
