@@ -6,12 +6,8 @@ use super::{
 use crate::circuit::algebraic_rewrite::einsum_nest_optimize;
 use crate::circuit::circuit_optimizer::OptimizationSettings;
 use crate::circuit::named_axes::axis_of_name;
-use crate::circuit::{
-    deep_map_op, flat_concat, flat_concat_back, visit_circuit, visit_circuit_with_parents,
-    visit_circuit_with_parents_fallible, Concat, EinsumAxes,
-};
+use crate::circuit::{deep_map_op, visit_circuit, visit_circuit_with_parents, Concat, EinsumAxes};
 use crate::circuit::{deep_map_unwrap, named_axes::set_named_axes, CircResult, ModuleNode};
-use crate::hashmaps::FxHashMap as HashMap;
 use crate::set_cover::set_cover;
 use crate::tensor_util::TensorAxisIndex;
 use crate::{all_imports::TensorIndex, pyo3_prelude::*};
@@ -263,7 +259,7 @@ pub fn batch_einsum(einsum: &Einsum, context: &mut OptimizationContext) -> CircR
     let out = deep_map_op(out_still_symbols, |c| {
         if let Some(sym) = c.as_symbol() && let Some(p) = einsum_args_named_axes
             .iter()
-            .position(|(s, ints)| sym.uuid==s.as_symbol().unwrap().uuid)
+            .position(|(s, _ints)| sym.uuid==s.as_symbol().unwrap().uuid)
         {
             return Some(einsum.args[p].0.clone());
         }
