@@ -6,11 +6,11 @@ use crate::hashmaps::{AHashSet as HashSet, FxHashMap as HashMap};
 use crate::pycall;
 use pyo3::{pyfunction, PyObject, Python};
 
-pub fn filter_nodes(circuit: CircuitRc, filter: &dyn Fn(&Circuit) -> bool) -> HashSet<CircuitRc> {
+pub fn filter_nodes(circuit: CircuitRc, filter: &dyn Fn(CircuitRc) -> bool) -> HashSet<CircuitRc> {
     let mut result: HashSet<CircuitRc> = HashSet::new();
-    visit_circuit(&circuit, |circuit| {
-        if filter(circuit) {
-            result.insert(circuit.clone().rc());
+    visit_circuit(circuit, |circuit| {
+        if filter(circuit.clone()) {
+            result.insert(circuit);
         }
     });
     result
@@ -23,12 +23,12 @@ pub fn filter_nodes_py(
     filter: PyObject,
 ) -> Result<HashSet<CircuitRc>, CircuitConstructionError> {
     let mut result: HashSet<CircuitRc> = HashSet::new();
-    let err = visit_circuit_fallable(&circuit, |circuit| {
+    let err = visit_circuit_fallable(circuit, |circuit| {
         let filter_result: Result<_, CircuitConstructionError> =
-            pycall!(filter, (circuit.clone().rc(),), CircuitConstructionError);
+            pycall!(filter, (circuit.clone(),), CircuitConstructionError);
         let filter_result = filter_result?;
         if filter_result {
-            result.insert(circuit.clone().rc());
+            result.insert(circuit);
         }
         Ok(())
     });
@@ -38,9 +38,8 @@ pub fn filter_nodes_py(
 #[pyfunction]
 #[pyo3(name = "replace_nodes")]
 pub fn replace_nodes_py(circuit: CircuitRc, map: HashMap<CircuitRc, CircuitRc>) -> CircuitRc {
-    deep_map_unwrap_preorder(&circuit, |x: &Circuit| -> CircuitRc {
-        let rc = x.clone().rc();
-        map.get(&rc).cloned().unwrap_or(rc)
+    deep_map_unwrap_preorder(circuit, |x: CircuitRc| -> CircuitRc {
+        map.get(&x).cloned().unwrap_or(x)
     })
 }
 
@@ -48,9 +47,9 @@ pub fn replace_nodes_py(circuit: CircuitRc, map: HashMap<CircuitRc, CircuitRc>)
 /// In pre-order so that high level nodes are replaced first
 pub fn replace_nodes(circuit: CircuitRc, map: &HashMap<HashBytes, CircuitRc>) -> CircuitRc {
     deep_map_op_context_preorder_stoppable(
-        &circuit,
-        &|x: &Circuit, _| {
-            let rc = x.clone().rc();
+        circuit.clone(),
+        &|x: CircuitRc, _| {
+            let rc = x;
             let result = map.get(&rc.info().hash).cloned();
             let stop = result.is_some();
             (result, stop)
@@ -68,9 +67,9 @@ pub fn deep_map_preorder_py(
     f: PyObject,
 ) -> Result<CircuitRc, CircuitConstructionError> {
     deep_map_preorder(
-        &circuit,
-        |x: &Circuit| -> Result<CircuitRc, CircuitConstructionError> {
-            pycall!(f, (x.clone().rc(),), CircuitConstructionError)
+        circuit,
+        |x: CircuitRc| -> Result<CircuitRc, CircuitConstructionError> {
+            pycall!(f, (x,), CircuitConstructionError)
         },
     )
 }
@@ -78,11 +77,11 @@ pub fn deep_map_preorder_py(
 #[pyfunction]
 #[pyo3(name = "deep_map")]
 pub fn deep_map_py(circuit: CircuitRc, f: PyObject) -> Result<CircuitRc, CircuitConstructionError> {
-    deep_map(&circuit, &|x: &Circuit| -> Result<
+    deep_map(circuit, &|x: CircuitRc| -> Result<
         CircuitRc,
         CircuitConstructionError,
     > {
-        pycall!(f, (x.clone().rc(),), CircuitConstructionError)
+        pycall!(f, (x,), CircuitConstructionError)
     })
 }
 
@@ -94,12 +93,11 @@ pub fn update_nodes_py(
     updater: PyObject,
 ) -> Result<CircuitRc, CircuitConstructionError> {
     let nodes = filter_nodes_py(circuit.clone(), matcher)?;
-    deep_map_preorder(&circuit, |x| {
-        let xcloned = x.clone().rc();
-        if nodes.contains(&xcloned) {
-            pycall!(updater, (xcloned,), CircuitConstructionError)
+    deep_map_preorder(circuit, |x| {
+        if nodes.contains(&x) {
+            pycall!(updater, (x,), CircuitConstructionError)
         } else {
-            Ok(xcloned)
+            Ok(x)
         }
     })
 }
@@ -120,21 +118,21 @@ pub fn path_get(circuit: CircuitRc, path: CircuitPath) -> Option<CircuitRc> {
 }
 
 pub fn update_path<F>(
-    circuit: &Circuit,
+    circuit: CircuitRc,
     path: &CircuitPath,
     updater: F,
 ) -> Result<CircuitRc, CircuitConstructionError>
 where
-    F: Fn(&Circuit) -> Result<CircuitRc, CircuitConstructionError>,
+    F: Fn(CircuitRc) -> Result<CircuitRc, CircuitConstructionError>,
 {
     fn recurse<F>(
-        circuit: &Circuit,
+        circuit: CircuitRc,
         path: &CircuitPath,
         path_idx: usize,
         updater: &F,
     ) -> Result<CircuitRc, CircuitConstructionError>
     where
-        F: Fn(&Circuit) -> Result<CircuitRc, CircuitConstructionError>,
+        F: Fn(CircuitRc) -> Result<CircuitRc, CircuitConstructionError>,
     {
         if path_idx == path.len() {
             return updater(circuit);
@@ -144,7 +142,7 @@ where
                 if i == path[path_idx] {
                     recurse(circuit, path, path_idx + 1, updater)
                 } else {
-                    Ok(circuit.clone().rc())
+                    Ok(circuit)
                 }
             })
             .map(|z| z.rc())
@@ -159,7 +157,7 @@ pub fn update_path_py(
     path: CircuitPath,
     updater: PyObject,
 ) -> Result<CircuitRc, CircuitConstructionError> {
-    update_path(&circuit, &path, |x| {
-        pycall!(updater, (x.clone().rc(),), CircuitConstructionError)
+    update_path(circuit, &path, |x| {
+        pycall!(updater, (x.clone(),), CircuitConstructionError)
     })
 }
