@@ -18,72 +18,75 @@ use std::hash::Hash;
 
 #[pyfunction]
 pub fn cast_circuit(circ: CircuitRc, device_dtype: &TorchDeviceDtype) -> CircuitRc {
-    deep_map_op_pre_new_children(&circ, |circ, children: &Vec<CircuitRc>| match circ {
-        Circuit::ArrayConstant(node) => {
-            if &TorchDeviceDtype::from_tensor(&node.value) != device_dtype {
-                Some(ArrayConstant::nrc(
-                    device_dtype.cast_tensor(node.value.clone()),
-                    node.name_cloned(),
-                ))
-            } else {
-                None
+    deep_map_op_pre_new_children(
+        circ.clone(),
+        |circ, children: &Vec<CircuitRc>| match &**circ {
+            Circuit::ArrayConstant(node) => {
+                if &TorchDeviceDtype::from_tensor(&node.value) != device_dtype {
+                    Some(ArrayConstant::nrc(
+                        device_dtype.cast_tensor(node.value.clone()),
+                        node.name_cloned(),
+                    ))
+                } else {
+                    None
+                }
             }
-        }
-        Circuit::Index(node) => {
-            if node
-                .index
-                .0
-                .iter()
-                .any(|x| matches!(x, TensorAxisIndex::Tensor(_)))
-            {
-                Some(Index::nrc(
-                    children[0].clone(),
-                    TensorIndex(
-                        node.index
-                            .0
-                            .iter()
-                            .map(|idx| match idx {
-                                TensorAxisIndex::Tensor(tensor) => TensorAxisIndex::Tensor(
-                                    TorchDeviceDtype {
-                                        device: device_dtype.device.clone(),
-                                        dtype: "int64".to_owned(),
-                                    }
-                                    .cast_tensor(tensor.clone()),
-                                ),
-                                _ => idx.clone(),
-                            })
-                            .collect(),
-                    ),
-                    node.name_cloned(),
-                ))
-            } else {
-                None
+            Circuit::Index(node) => {
+                if node
+                    .index
+                    .0
+                    .iter()
+                    .any(|x| matches!(x, TensorAxisIndex::Tensor(_)))
+                {
+                    Some(Index::nrc(
+                        children[0].clone(),
+                        TensorIndex(
+                            node.index
+                                .0
+                                .iter()
+                                .map(|idx| match idx {
+                                    TensorAxisIndex::Tensor(tensor) => TensorAxisIndex::Tensor(
+                                        TorchDeviceDtype {
+                                            device: device_dtype.device.clone(),
+                                            dtype: "int64".to_owned(),
+                                        }
+                                        .cast_tensor(tensor.clone()),
+                                    ),
+                                    _ => idx.clone(),
+                                })
+                                .collect(),
+                        ),
+                        node.name_cloned(),
+                    ))
+                } else {
+                    None
+                }
             }
-        }
-        _ => None,
-    })
-    .unwrap_or(circ.clone())
+            _ => None,
+        },
+    )
+    .unwrap_or(circ)
 }
 
 #[pyfunction]
 pub fn count_nodes(circuit: CircuitRc) -> usize {
     let mut result: usize = 0;
-    visit_circuit(&circuit, &mut |_x: &Circuit| result += 1);
+    visit_circuit(circuit, &mut |_x: CircuitRc| result += 1);
     result
 }
 
-pub fn hash_to_node(circuit: &Circuit) -> HashMap<HashBytes, CircuitRc> {
+pub fn hash_to_node(circuit: CircuitRc) -> HashMap<HashBytes, CircuitRc> {
     let mut result: HashMap<HashBytes, CircuitRc> = HashMap::new();
-    visit_circuit(circuit, &mut |x: &Circuit| {
-        result.insert(x.info().hash, x.clone().rc());
+    visit_circuit(circuit, &mut |x: CircuitRc| {
+        result.insert(x.info().hash, x.clone());
     });
     result
 }
 
-pub fn subcircuits(circuit: &Circuit) -> HashSet<CircuitRc> {
+pub fn subcircuits(circuit: CircuitRc) -> HashSet<CircuitRc> {
     let mut result: HashSet<CircuitRc> = HashSet::new();
-    visit_circuit(circuit, &mut |x: &Circuit| {
-        result.insert(x.clone().rc());
+    visit_circuit(circuit, &mut |x: CircuitRc| {
+        result.insert(x);
     });
     result
 }
@@ -91,7 +94,7 @@ pub fn subcircuits(circuit: &Circuit) -> HashSet<CircuitRc> {
 #[pyfunction]
 pub fn total_flops(circuit: CircuitRc) -> BigUint {
     let mut result: BigUint = BigUint::from(0usize);
-    visit_circuit(&circuit, &mut |x: &Circuit| result += x.self_flops());
+    visit_circuit(circuit, &mut |x: CircuitRc| result += x.self_flops());
     result
 }
 
@@ -107,8 +110,8 @@ pub fn total_flops_cached(circuit: CircuitRc, cache: &mut HashMap<HashBytes, Big
 #[pyfunction]
 pub fn total_arrayconstant_size(circuit: CircuitRc) -> BigUint {
     let mut result: BigUint = BigUint::from(0usize);
-    visit_circuit(&circuit, &mut |x: &Circuit| {
-        if let Circuit::ArrayConstant(x) = x {
+    visit_circuit(circuit, &mut |x: CircuitRc| {
+        if let Circuit::ArrayConstant(x) = &**x {
             result += x.info().numel();
         }
     });
@@ -118,8 +121,8 @@ pub fn total_arrayconstant_size(circuit: CircuitRc) -> BigUint {
 #[pyfunction]
 pub fn sum_of_node_sizes(circuit: CircuitRc) -> BigUint {
     let mut result: BigUint = BigUint::from(0usize);
-    visit_circuit(&circuit, &mut |x: &Circuit| {
-        if !matches!(x, Circuit::ArrayConstant(_)) {
+    visit_circuit(circuit, &mut |x: CircuitRc| {
+        if !matches!(&**x, Circuit::ArrayConstant(_)) {
             result += x.info().numel();
         }
     });
@@ -141,9 +144,9 @@ pub fn sum_of_node_sizes_cached(
 #[pyfunction]
 pub fn get_leaves(circuit: CircuitRc) -> Vec<CircuitRc> {
     let mut result: Vec<CircuitRc> = vec![];
-    visit_circuit(&circuit, &mut |c: &Circuit| {
+    visit_circuit(circuit, &mut |c: CircuitRc| {
         if c.children().count() == 0 {
-            result.push(c.clone().rc());
+            result.push(c);
         }
     });
     result
@@ -152,8 +155,8 @@ pub fn get_leaves(circuit: CircuitRc) -> Vec<CircuitRc> {
 #[pyfunction]
 pub fn get_all_einsum_specs(circuit: CircuitRc) -> Vec<EinsumSpec> {
     let mut result: Vec<EinsumSpec> = vec![];
-    visit_circuit(&circuit, |c| {
-        if let Circuit::Einsum(node) = c {
+    visit_circuit(circuit, |c| {
+        if let Circuit::Einsum(node) = &**c {
             result.push(node.get_spec());
         }
     });
@@ -164,7 +167,7 @@ pub fn get_all_einsum_specs(circuit: CircuitRc) -> Vec<EinsumSpec> {
 /// children first
 pub fn toposort_circuit(circuit: CircuitRc) -> Vec<CircuitRc> {
     let mut num_refs: HashMap<CircuitRc, usize> = HashMap::new();
-    visit_circuit(&circuit, |c| {
+    visit_circuit(circuit.clone(), |c| {
         for child in c.children() {
             *num_refs.entry(child).or_insert(0) += 1;
         }
@@ -185,18 +188,18 @@ pub fn toposort_circuit(circuit: CircuitRc) -> Vec<CircuitRc> {
     result
 }
 
-pub fn deep_map_pass_up<F, T>(circuit: &Circuit, mut f: F) -> (CircuitRc, T)
+pub fn deep_map_pass_up<F, T>(circuit: CircuitRc, mut f: F) -> (CircuitRc, T)
 where
     T: Clone,
-    F: FnMut(&Circuit, Vec<T>) -> (CircuitRc, T),
+    F: FnMut(CircuitRc, Vec<T>) -> (CircuitRc, T),
 {
-    let topo = toposort_circuit(circuit.clone().rc());
+    let topo = toposort_circuit(circuit.clone());
     let mut pass_ups: HashMap<CircuitRc, T> = HashMap::new();
     for c in topo {
         let passed_up = c.children().map(|c| pass_ups[&c].clone()).collect();
-        let (new, pass_here) = f(&c, passed_up);
+        let (new, pass_here) = f(c.clone(), passed_up);
         pass_ups.insert(c.clone(), pass_here.clone());
-        if &**c == circuit {
+        if c == circuit {
             return (new, pass_here);
         }
     }
@@ -220,11 +223,8 @@ where
                 .push(pass);
         }
     }
-    deep_map_unwrap(&circuit, |c| {
-        make_circuit(
-            c.clone().rc(),
-            passed_down.get(&c.clone().rc()).unwrap_or(&vec![]),
-        )
+    deep_map_unwrap(circuit, |c| {
+        make_circuit(c.clone(), passed_down.get(&c).unwrap_or(&vec![]))
     })
 }
 
@@ -284,7 +284,7 @@ where
 
 #[pyfunction]
 pub fn replace_all_randn_seeded(circuit: CircuitRc) -> CircuitRc {
-    deep_map_unwrap(&circuit, |node| match node {
+    deep_map_unwrap(circuit, |node| match &**node {
         Circuit::ArrayConstant(ac) => ArrayConstant::randn_seeded(
             ac.info().shape.clone(),
             ac.name_cloned(),
@@ -312,13 +312,13 @@ pub fn replace_all_randn_seeded(circuit: CircuitRc) -> CircuitRc {
             ),
             index.name_cloned(),
         ),
-        _ => node.clone().rc(),
+        _ => node.clone(),
     })
 }
 
 pub fn has_leaf_symbols(circuit: CircuitRc, allowed: &HashSet<HashBytes>) -> bool {
     let mut result = false;
-    visit_circuit(&circuit, |circuit| {
+    visit_circuit(circuit, |circuit| {
         if circuit.as_symbol().is_some() && !allowed.contains(&circuit.info().hash) {
             println!(
                 "found symbol {:?} allowed {:?}",
@@ -327,7 +327,7 @@ pub fn has_leaf_symbols(circuit: CircuitRc, allowed: &HashSet<HashBytes>) -> boo
             );
             result = true;
         }
-        if let Circuit::ModuleNode(mn) = circuit {
+        if let Circuit::ModuleNode(mn) = &**circuit {
             let inner_allowed: HashSet<HashBytes> = allowed
                 .iter()
                 .cloned()
