@@ -39,7 +39,7 @@ pub type EinsumAxes = Sv<[EinInt; 8]>;
 /// our Einsum supports diag output
 /// this means `a->aa` is valid and produces a tensor with the input on the diagonal
 /// also `aa->aa` only copies the diagonal, and ignores the rest
-#[pyclass(unsendable, extends=PyCircuitBase)]
+#[pyclass(extends=PyCircuitBase)]
 #[derive(Debug, Clone, PyClassDeriv)]
 pub struct Einsum {
     pub args: EinsumArgs,
@@ -465,7 +465,7 @@ impl Einsum {
 }
 
 /// Add supports broadcasting
-#[pyclass(unsendable, extends=PyCircuitBase)]
+#[pyclass(extends=PyCircuitBase)]
 #[derive(Debug, Clone, PyClassDeriv)]
 pub struct Add {
     #[pyo3(get)]
@@ -661,7 +661,7 @@ impl CircuitNodeAutoName for Add {
     }
 }
 
-#[pyclass(unsendable, extends=PyCircuitBase)]
+#[pyclass(extends=PyCircuitBase)]
 #[derive(Debug, Clone, PyClassDeriv)]
 pub struct Rearrange {
     #[pyo3(get)]
@@ -877,7 +877,7 @@ impl Rearrange {
 /// and each 1-d tensor is iterated independently, unlike torch or numpy
 /// tensor indices which are iterated together.
 
-#[pyclass(unsendable, extends=PyCircuitBase)]
+#[pyclass(extends=PyCircuitBase)]
 #[derive(Debug, Clone, PyClassDeriv)]
 pub struct Index {
     #[pyo3(get)]
@@ -1081,7 +1081,7 @@ impl Index {
     }
 }
 
-#[pyclass(unsendable, extends=PyCircuitBase)]
+#[pyclass(extends=PyCircuitBase)]
 #[derive(Debug, Clone)]
 pub struct Concat {
     pub nodes: Vec<CircuitRc>,
@@ -1254,7 +1254,7 @@ impl Concat {
 /// but index is considered dimwise
 ///
 /// right now rewrites only work with slices, maybe will support others later
-#[pyclass(unsendable, extends=PyCircuitBase)]
+#[pyclass(extends=PyCircuitBase)]
 #[derive(Debug, Clone, PyClassDeriv)]
 pub struct Scatter {
     #[pyo3(get)]
