@@ -1,4 +1,5 @@
-use std::{collections::HashMap, iter::zip};
+use crate::hashmaps::FxHashMap as HashMap;
+use std::iter::zip;
 
 use crate::pyo3_prelude::*;
 use num_bigint::BigUint;
