@@ -31,10 +31,7 @@ macro_rules! circuit_node_auto_leaf_impl {
             vec![]
         }
 
-        fn map_children_enumerate<'a, F, E>(
-            &'a self,
-            _f: F,
-        ) -> Result<Self, CircuitConstructionError>
+        fn map_children_enumerate<F, E>(&self, _f: F) -> Result<Self, CircuitConstructionError>
         where
             CircuitConstructionError: From<E>,
             F: FnMut(usize, CircuitRc) -> Result<CircuitRc, E>,
