@@ -58,8 +58,8 @@ pub fn is_close(a: Tensor, b: Tensor) -> Result<PyObject, PyErr> {
 }
 
 pub fn assert_is_close(a: CircuitRc, b: CircuitRc) {
-    let a_ten = evaluate_fn(&a).unwrap();
-    let b_ten = evaluate_fn(&b).unwrap();
+    let a_ten = evaluate_fn(a.clone()).unwrap();
+    let b_ten = evaluate_fn(b.clone()).unwrap();
     let close_result = is_close(a_ten.clone(), b_ten.clone());
     if close_result.is_err() {
         println!("{:?}", close_result);
@@ -77,7 +77,7 @@ pub fn opt_eval_each_subcircuit_until_fail(circuit: CircuitRc, settings: Optimiz
     let topo = toposort_circuit(circuit.clone());
     for circ in topo {
         if circ.info().numel() < max_numel {
-            let reference = evaluate_fn(&circ).unwrap();
+            let reference = evaluate_fn(circ.clone()).unwrap();
             let optimized = optimize_and_evaluate(circ.clone(), settings);
             if let Ok(optimized) = optimized {
                 let close_result = is_close(reference, optimized);
