@@ -1,13 +1,14 @@
+use crate::circuit::batching::batch_einsum;
 use crate::circuit::generalfunction_rewrite::generalfunction_evaluate_simple;
+use crate::circuit::module_rewrite::elim_empty_module;
 use crate::circuit::scatter_rewrite::concat_to_scatter;
-use crate::hashmaps::{AHashSet as HashSet, FxHashMap as HashMap};
-
 use crate::circuit::{
     deep_map_op_context, generalfunction_merge_inverses,
     generalfunction_special_case_simplification,
 };
+use crate::hashmaps::{AHashSet as HashSet, FxHashMap as HashMap};
 use crate::util::{apply_fn_until_none, apply_fn_until_same, mapping_until_end, AsOp};
-use crate::{pyo3_prelude::*, timed_value};
+use crate::{pyo3_prelude::*, timed, timed_value};
 use num_bigint::BigUint;
 
 use super::{
@@ -155,6 +156,10 @@ pub fn compiler_simp_step(
             ];
             simp(node, fns, context)
         }
+        Circuit::ModuleNode(node) => {
+            let fns: &[(&'static str, &dyn Fn(_) -> _)] = &[l_wrap!(&elim_empty_module)];
+            simp(node, fns, context)
+        }
         _ => None,
     }
 }
@@ -169,23 +174,23 @@ pub fn compiler_simp_step(
 #[pyo3(name = "compiler_simp")]
 pub fn compiler_simp_py(circ: CircuitRc) -> CircuitRc {
     let mut context = Default::default();
-    compiler_simp(&circ, &mut context)
+    compiler_simp(circ, &mut context)
 }
 
-pub fn compiler_simp(circ: &Circuit, opt_context: &mut OptimizationContext) -> CircuitRc {
+pub fn compiler_simp(circ: CircuitRc, opt_context: &mut OptimizationContext) -> CircuitRc {
     /// check if any new children have not been simplified yet, and simplify them if so
     fn simplify_changed_descendants(
-        circ: &Circuit,
+        circ: CircuitRc,
         context: &mut OptimizationContext,
     ) -> Option<CircuitRc> {
         // if let Some(changed) = compiler_simp_step(circ){
         //     return simplify_changed_descendants(circ, simplified)
         // }
-        circ.map_children_op(&mut |x: &Circuit| {
+        circ.map_children_op(&mut |x: CircuitRc| {
             if context.cache.simplified.contains_key(&x.info().hash) {
                 None
             } else {
-                Some(fully_simplify(x.clone().rc(), context))
+                Some(fully_simplify(x, context))
             }
         })
         .map(|c| c.rc())
@@ -196,11 +201,11 @@ pub fn compiler_simp(circ: &Circuit, opt_context: &mut OptimizationContext) -> C
             return result.clone();
         }
         let mut result: CircuitRc = circ
-            .map_children_unwrap(&mut |x: &Circuit| fully_simplify(x.clone().rc(), context))
+            .map_children_unwrap(&mut |x: CircuitRc| fully_simplify(x, context))
             .rc();
         for iter_count in 0.. {
             match compiler_simp_step(&result, context) {
-                Some(r) => result = simplify_changed_descendants(&r, context).unwrap_or(r),
+                Some(r) => result = simplify_changed_descendants(r.clone(), context).unwrap_or(r),
                 None => break,
             }
             if iter_count > 50 {
@@ -217,33 +222,35 @@ pub fn compiler_simp(circ: &Circuit, opt_context: &mut OptimizationContext) -> C
             .insert(circ.info().hash, result.clone());
         result
     }
-    fully_simplify(circ.clone().rc(), opt_context)
+    fully_simplify(circ, opt_context)
 }
 
 #[pyfunction]
 pub fn compiler_simp_until_same(circ: CircuitRc) -> CircuitRc {
     let mut context = Default::default();
 
-    apply_fn_until_same(&circ, |x: &CircuitRc| compiler_simp(x, &mut context))
+    apply_fn_until_same(&circ, |x: &CircuitRc| {
+        compiler_simp(x.clone(), &mut context)
+    })
 }
 
 #[pyfunction]
 pub fn deep_push_down_index(circ: CircuitRc, min_size: Option<usize>) -> CircuitRc {
-    deep_map_unwrap_preorder(&circ, |circ| {
+    deep_map_unwrap_preorder(circ, |circ| {
         if min_size.is_none()
             || circ
                 .children()
-                .chain(std::iter::once(circ.clone().rc()))
+                .chain(std::iter::once(circ.clone()))
                 .any(|z| z.info().numel() >= BigUint::from(min_size.unwrap()))
         {
-            circ.map_or_clone(&|index: &Index| {
+            (**circ).map_or_clone(&|index: &Index| {
                 let fused = apply_fn_until_none(index, index_fuse);
                 index_elim_identity(&fused).unwrap_or_else(|| {
                     push_down_index(&fused).unwrap_or_else(|| fused.clone().rc())
                 })
             })
         } else {
-            circ.clone().rc()
+            circ
         }
     })
 }
@@ -256,7 +263,7 @@ pub fn deep_push_down_index(circ: CircuitRc, min_size: Option<usize>) -> Circuit
 pub fn deep_heuristic_nest_adds(circ: CircuitRc) -> CircuitRc {
     let circ = deep_canonicalize(circ, &mut Default::default());
     let mut seen_adds: HashSet<Add> = HashSet::new();
-    visit_circuit(&circ, &mut |c: &Circuit| {
+    visit_circuit(circ.clone(), &mut |c: CircuitRc| {
         if let Some(add) = c.as_add() {
             seen_adds.insert(add.clone());
         }
@@ -300,8 +307,8 @@ pub fn deep_heuristic_nest_adds(circ: CircuitRc) -> CircuitRc {
         mapping.insert(sup, new_sup.clone());
     }
 
-    deep_map_unwrap_preorder(&circ, |c| {
-        c.map_or_clone(|add: &Add| {
+    deep_map_unwrap_preorder(circ, |c| {
+        (**c).map_or_clone(|add: &Add| {
             let add = mapping_until_end(add, &mapping);
 
             if add.info().numel() > BigUint::from(100_000_000usize) {
@@ -329,23 +336,23 @@ pub fn add_nest_ltr_py(add: Add) -> Add {
 
 #[pyfunction]
 pub fn deep_pull_concat_messy(circuit: CircuitRc, min_size: Option<usize>) -> CircuitRc {
-    deep_map_unwrap(&circuit, &|x: &Circuit| {
+    deep_map_unwrap(circuit, &|x: CircuitRc| {
         if min_size.is_none()
             || x.children()
-                .chain(std::iter::once(x.clone().rc()))
+                .chain(std::iter::once(x.clone()))
                 .any(|z| z.info().numel() >= BigUint::from(min_size.unwrap()))
         {
-            match x {
+            match &**x {
                 Circuit::Add(add) => add.and_then_or_clone(add_pull_concat),
                 Circuit::GeneralFunction(func) => {
                     func.and_then_or_clone(generalfunction_pull_concat)
                 }
                 Circuit::Einsum(einsum) => einsum.and_then_or_clone(einsum_pull_concat),
                 Circuit::Concat(concat) => concat.and_then_or_clone(concat_fuse),
-                _ => x.clone().rc(),
+                _ => x.clone(),
             }
         } else {
-            x.clone().rc()
+            x
         }
     })
 }
@@ -355,7 +362,7 @@ pub fn deep_pull_concat(circuit: CircuitRc, min_size: Option<usize>) -> CircuitR
     let mut cache = Default::default();
     let pulled = deep_pull_concat_messy(circuit, min_size);
     apply_fn_until_same(&pulled, |x: &CircuitRc| {
-        deep_push_down_index(compiler_simp(x, &mut cache), min_size)
+        deep_push_down_index(compiler_simp(x.clone(), &mut cache), min_size)
     })
 }
 
@@ -367,20 +374,24 @@ pub fn deep_optimize_einsums_py(circ: CircuitRc) -> CircuitRc {
 
 pub fn deep_optimize_einsums(circ: CircuitRc, context: &mut OptimizationContext) -> CircuitRc {
     deep_map_op_context(
-        &circ,
-        &|x: &Circuit, context: &mut OptimizationContext| match x {
+        circ.clone(),
+        &|x: CircuitRc, context: &mut OptimizationContext| match &**x {
             Circuit::Einsum(ein) => {
                 let (result, took) = timed_value!(einsum_nest_optimize(ein, context));
-                let result = result?;
-                if took.as_millis() > 10 {
-                    context.cache.slow_einsum_log.push(ein.get_spec());
+                if result.is_none() {
+                    Some(timed!(batch_einsum(ein, context).unwrap()))
+                } else {
+                    let result = result?;
+                    if context.settings.log_slow_einsums && took.as_millis() > 10 {
+                        context.cache.slow_einsum_log.push(ein.get_spec());
+                    }
+                    Some(result.rc())
                 }
-                Some(result.rc())
             }
             _ => None,
         },
         context,
         &mut HashMap::new(),
     )
-    .unwrap_or(circ.clone())
+    .unwrap_or(circ)
 }
