@@ -163,7 +163,7 @@ impl GeneralFunctionSpec {
     }
 }
 
-#[pyclass(unsendable, extends=PyCircuitBase)]
+#[pyclass(extends=PyCircuitBase)]
 #[derive(Debug, Clone, PyClassDeriv)]
 pub struct GeneralFunction {
     #[pyo3(get)]
