@@ -27,6 +27,7 @@ use py_circuit_items::circuit_rust_to_py;
 use pyo3::{exceptions, pyclass::CompareOp};
 use std::collections::BTreeMap;
 use std::hash::Hash;
+use std::sync::Arc;
 use std::{
     iter::zip,
     ops::{Deref, DerefMut},
@@ -298,7 +299,7 @@ pub trait CircuitNode: CircuitNodePrivate {
     where
         Self: Into<Circuit>,
     {
-        CircuitRc(Rc::new(self.c()))
+        CircuitRc(Arc::new(self.c()))
     }
 
     fn compiler_repr(&self) -> String
@@ -622,7 +623,7 @@ macro_rules! circuit_node_extra_impl {
                 // kinda awkward clone... (but probably basically free)
                 (
                     self.clone(),
-                    $crate::circuit::PyCircuitBase(::std::rc::Rc::new(self.c())),
+                    $crate::circuit::PyCircuitBase(::std::sync::Arc::new(self.c())),
                 )
                     .into()
             }
@@ -1016,7 +1017,7 @@ pub enum CircuitConstructionError {
     NoEquivalentExplicitlyComputable {},
 
     #[error("python error {py_err:?}")]
-    PythonError { py_err: Rc<PyErr> },
+    PythonError { py_err: Arc<PyErr> },
 }
 
 impl From<CircuitConstructionError> for PyErr {
@@ -1028,7 +1029,7 @@ impl From<CircuitConstructionError> for PyErr {
 impl From<PyErr> for CircuitConstructionError {
     fn from(py_err: PyErr) -> Self {
         CircuitConstructionError::PythonError {
-            py_err: Rc::new(py_err),
+            py_err: Arc::new(py_err),
         }
     }
 }
@@ -1040,7 +1041,7 @@ pub enum TensorEvalError {
     #[error("not explicitly computable: {circuit:?})")]
     NotExplicitlyComputable { circuit: CircuitRc },
     #[error("python error {py_err:?}")]
-    PythonError { py_err: Rc<PyErr> }, // PyErr doesn't have .clone, so Rc-ing
+    PythonError { py_err: Arc<PyErr> }, // PyErr doesn't have .clone, so Rc-ing
     #[error("incompatible dtype circ:{circ:?} passed:{passed:?}")]
     DeviceDtypeError {
         circ: TorchDeviceDtypeOp,
@@ -1059,13 +1060,13 @@ impl From<TensorEvalError> for PyErr {
 impl From<PyErr> for TensorEvalError {
     fn from(py_err: PyErr) -> Self {
         TensorEvalError::PythonError {
-            py_err: Rc::new(py_err),
+            py_err: Arc::new(py_err),
         }
     }
 }
 
 #[derive(Clone, Debug, PartialEq, Eq, Hash, PartialOrd, Ord)]
-pub struct CircuitRc(Rc<Circuit>);
+pub struct CircuitRc(Arc<Circuit>);
 
 pub fn make_children_zero<T: CircuitNode>(circuit: &T) -> T {
     circuit.map_children_unwrap(&mut |child: CircuitRc| {
@@ -1446,14 +1447,14 @@ impl<T: CircuitNode + Into<Circuit>> From<T> for CircuitRc {
     }
 }
 
-impl From<Rc<Circuit>> for CircuitRc {
-    fn from(x: Rc<Circuit>) -> Self {
+impl From<Arc<Circuit>> for CircuitRc {
+    fn from(x: Arc<Circuit>) -> Self {
         CircuitRc(x)
     }
 }
 
 impl Deref for CircuitRc {
-    type Target = Rc<Circuit>;
+    type Target = Arc<Circuit>;
 
     fn deref(&self) -> &Self::Target {
         &self.0
@@ -1466,12 +1467,12 @@ impl DerefMut for CircuitRc {
     }
 }
 
-#[pyclass(unsendable, subclass, name = "Circuit")]
+#[pyclass(subclass, name = "Circuit")]
 #[derive(Clone, Debug)]
-pub struct PyCircuitBase(Rc<Circuit>);
+pub struct PyCircuitBase(Arc<Circuit>);
 
 impl Deref for PyCircuitBase {
-    type Target = Rc<Circuit>;
+    type Target = Arc<Circuit>;
 
     fn deref(&self) -> &Self::Target {
         &self.0
