@@ -249,7 +249,7 @@ where
     deep_map_fallible_pre_new_children(circuit, recursor)
 }
 
-#[pyclass(unsendable)]
+#[pyclass]
 #[derive(Debug, Clone, PyClassDeriv, Hash, PartialEq, Eq)]
 pub struct ModuleNodeSpec {
     #[pyo3(get)]
@@ -530,7 +530,7 @@ impl ModuleNodeSpec {
     }
 }
 
-#[pyclass(unsendable)]
+#[pyclass]
 #[derive(Debug, Clone, PyClassDeriv, Hash, PartialEq, Eq)]
 pub struct ModuleNodeArgSpec {
     #[pyo3(get)]
@@ -552,7 +552,7 @@ impl ModuleNodeArgSpec {
     }
 }
 
-#[pyclass(unsendable, extends=PyCircuitBase)]
+#[pyclass(extends=PyCircuitBase)]
 #[derive(Debug, Clone, PyClassDeriv)]
 pub struct ModuleNode {
     #[pyo3(get)]
