@@ -135,27 +135,27 @@ pub fn get_axis_leaves(
     let mut result_axis: HashMap<CircuitRc, usize> = Default::default();
     let mut result_no_axis: HashSet<CircuitRc> = Default::default();
     let _names_to_remove = Some(HashSet::from([name.clone()]));
-    visit_circuit_with_parents_fallible(&circ_named_axes, |x, parents| {
+    visit_circuit_with_parents_fallible(circ_named_axes.clone(), |x, parents| {
         // println!("child");
         // x.compiler_print();
         // println!("parents");
         // for parent in parents {
         //     parent.compiler_print();
         // }
-        if let Some(i) = axis_of_name(x,&name) && !x.children().any(|child|axis_of_name(&child, &name).is_some())
+        if let Some(i) = axis_of_name(&x,&name) && !x.children().any(|child|axis_of_name(&child, &name).is_some())
         {
             // if  error_on_removable_axis_leaf && get_removable_axes(&x.clone().rc()).contains(&i){
             //     return Err(CircuitConstructionError::ExpandingRemovableAxisUnfortunateError {  })
             // }
-            result_axis.insert(x.clone().rc(), i);
+            result_axis.insert(x.clone(), i);
         }
 
-        if axis_of_name(x, &name).is_none()
+        if axis_of_name(&x, &name).is_none()
             && parents
                 .iter()
                 .any(|child| axis_of_name(child, &name).is_some())
         {
-            result_no_axis.insert(x.clone().rc());
+            result_no_axis.insert(x);
         }
         Ok::<(), CircuitConstructionError>(())
     })?;
@@ -163,8 +163,9 @@ pub fn get_axis_leaves(
 }
 
 /// remove all instances of this axis name from circuit
-pub fn deep_strip_axis_names(circuit: &Circuit, names: &Option<HashSet<String>>) -> CircuitRc {
+pub fn deep_strip_axis_names(circuit: CircuitRc, names: &Option<HashSet<String>>) -> CircuitRc {
     deep_map_unwrap(circuit, |x| {
+        let mut result = (**x).clone();
         let axis_names: NamedAxes = if let Some(blacklist) = &names {
             x.info()
                 .named_axes
@@ -175,7 +176,6 @@ pub fn deep_strip_axis_names(circuit: &Circuit, names: &Option<HashSet<String>>)
         } else {
             BTreeMap::new()
         };
-        let mut result = x.clone();
         result.info_mut().named_axes = axis_names;
         result.init_info().unwrap().rc()
     })
