@@ -16,6 +16,7 @@ use regex::Regex;
 use std::iter::zip;
 use std::rc::Rc;
 use std::str::FromStr;
+use std::sync::Arc;
 use uuid::Uuid;
 
 #[pyfunction(
@@ -183,7 +184,9 @@ pub fn parse_compiler_repr_bijection(
                 } else {
                     ArrayConstant::from_hash_prefix(ps.name.clone(), &ps.extra)
                         .map(|z| z.rc())
-                        .map_err(|e| CircuitConstructionError::PythonError { py_err: Rc::new(e) })
+                        .map_err(|e| CircuitConstructionError::PythonError {
+                            py_err: Arc::new(e),
+                        })
                 }
             }
             "Scalar" => Ok(ScalarConstant::nrc(
