@@ -184,7 +184,7 @@ pub static PY_EINOPS: GILLazyPy<PyEinops> = GILLazyPy::new_py(|py| {
     }
 });
 
-pub static HASH_TENSOR: GILLazyPy<PyObject> = GILLazyPy::new_py(|py| {
+pub static HASH_TENSOR: GILLazyPy<(PyObject, PyObject)> = GILLazyPy::new_py(|py| {
     let module = PyModule::from_code(
         py,
         include_str!(concat!(env!("CARGO_MANIFEST_DIR"), "/tensor_hash.py")),
@@ -192,7 +192,10 @@ pub static HASH_TENSOR: GILLazyPy<PyObject> = GILLazyPy::new_py(|py| {
         "tensor_hash",
     )
     .unwrap();
-    module.getattr("hash_tensor").unwrap().into()
+    (
+        module.getattr("hash_tensor").unwrap().into(),
+        module.getattr("pop_cuda_context").unwrap().into(),
+    )
 });
 
 #[macro_export]
@@ -386,6 +389,7 @@ impl Tensor {
                 shape: self.shape.clone(),
                 hash: Python::with_gil(|py| {
                     HASH_TENSOR
+                        .0
                         .call(py, (self.tensor.clone(),), None)
                         .unwrap()
                         .extract(py)
