@@ -25,6 +25,8 @@ pub struct OpSize(pub u64);
 pub type OpShape = Sv<[OpSize; 6]>;
 
 impl OpSize {
+    pub const SHIFT: usize = 63;
+    pub const NONE: OpSize = OpSize(1 << Self::SHIFT);
     // sometimes into/from can't do type interference, so we have aliases
     fn t(self) -> Option<usize> {
         self.into()
@@ -34,10 +36,10 @@ impl OpSize {
         val.into()
     }
     pub fn is_some(&self) -> bool {
-        self.0 >> 63 == 0
+        self.0 >> Self::SHIFT == 0
     }
     pub fn is_none(&self) -> bool {
-        self.0 >> 63 != 0
+        self.0 >> Self::SHIFT != 0
     }
     pub fn unwrap(self) -> u64 {
         assert!(self.is_some());
@@ -55,9 +57,9 @@ impl From<Option<usize>> for OpSize {
     fn from(x: Option<usize>) -> Self {
         assert!(std::mem::size_of::<usize>() <= std::mem::size_of::<u64>()); // optimized out I assume
         match x {
-            None => OpSize(1 << 63),
+            None => Self::NONE,
             Some(value) => {
-                assert_eq!(value >> 63, 0);
+                assert_eq!(value >> Self::SHIFT, 0);
                 OpSize(value as u64)
             }
         }
@@ -66,7 +68,7 @@ impl From<Option<usize>> for OpSize {
 
 impl From<OpSize> for Option<usize> {
     fn from(x: OpSize) -> Self {
-        if x.0 >> 63 == 1 {
+        if x.0 >> OpSize::SHIFT == 1 {
             None
         } else {
             Some(x.0 as usize)
@@ -177,6 +179,24 @@ impl RearrangeSpec {
             int_sizes: sv![OpSize::from(None); rank],
         }
     }
+
+    #[staticmethod]
+    pub fn prepend_batch_shape(new_shape: Shape, old_rank: usize) -> Self {
+        RearrangeSpec {
+            input_ints: (new_shape.len()..new_shape.len() + old_rank)
+                .map(|i| sv![i as u8])
+                .collect(),
+            output_ints: (0..new_shape.len() + old_rank)
+                .map(|i| sv![i as u8])
+                .collect(),
+            int_sizes: new_shape
+                .iter()
+                .map(|i| OpSize(*i as u64))
+                .chain(std::iter::repeat(OpSize::NONE).take(old_rank))
+                .collect(),
+        }
+    }
+
     pub fn to_py_rearrange_spec(&self, input_shape: Shape) -> PyObject {
         Python::with_gil(|py| {
             PY_CIRCUIT_ITEMS
@@ -204,6 +224,14 @@ impl RearrangeSpec {
     pub fn ints_in_out(&self) -> HashSet<EinInt> {
         self.ints_in_out_it().collect()
     }
+    #[staticmethod]
+    pub fn new_permute(permutation: Vec<usize>) -> RearrangeSpec {
+        RearrangeSpec::new(
+            (0..permutation.len()).map(|i| sv![i as u8]).collect(),
+            permutation.iter().map(|i| sv![*i as u8]).collect(),
+            sv![OpSize::NONE;permutation.len()],
+        )
+    }
     pub fn is_permute(&self) -> bool {
         // because there are no squeezes, all we need is no splits/joins and equal input and output rank
         self.input_ints.iter().all(|x| x.len() == 1)
