@@ -339,7 +339,7 @@ impl TensorAxisIndex {
                 Some(result)
             }
             TensorAxisIndex::Slice(slice) => {
-                let uslice: Option<USlice> = slice.clone().into();
+                let uslice: Option<USlice> = (*slice).into();
                 let uslice = uslice?;
                 let mut result = vec![None; sections.len()];
                 let mut did_any = false;
@@ -611,28 +611,28 @@ impl TensorIndex {
         };
         Ok(TensorIndex(
             string
-                .strip_prefix("[")
+                .strip_prefix('[')
                 .ok_or(er.clone())?
-                .strip_suffix("]")
+                .strip_suffix(']')
                 .ok_or(er.clone())?
-                .split(",")
+                .split(',')
                 .map(str::trim)
                 .filter(|x| !x.is_empty())
                 .map(|x| {
                     let num_er = CircuitConstructionError::ParsingNumberFail {
-                        string: format!("{}", x),
+                        string: x.to_string(),
                     };
                     if let Ok(int) = x.parse::<i64>() {
                         return Ok(TensorAxisIndex::Single(int));
                     }
-                    if let Some(no_t) = x.strip_prefix("t") {
+                    if let Some(no_t) = x.strip_prefix('t') {
                         return tensor_from_hash_prefix(no_t)
-                            .map(|z| TensorAxisIndex::Tensor(z))
+                            .map(TensorAxisIndex::Tensor)
                             .map_err(|_e| CircuitConstructionError::TensorHashNotFound {
                                 hash: no_t.to_owned(),
                             });
                     }
-                    if let [start, end] = x.split(":").collect::<Vec<_>>()[..] {
+                    if let [start, end] = x.split(':').collect::<Vec<_>>()[..] {
                         return Ok(TensorAxisIndex::Slice(Slice {
                             start: if start.is_empty() {
                                 None
@@ -646,7 +646,7 @@ impl TensorIndex {
                             },
                         }));
                     }
-                    return Err(er.clone());
+                    Err(er.clone())
                 })
                 .collect::<Result<Vec<_>, CircuitConstructionError>>()?,
         ))
